<?php
/**
 * The Odin_Driver:: class provides a common abstracted interface to the
 * various data source drivers. It includes functions for searching,
 * adding, removing, and modifying data entries.
 *
 * $Horde: odin/lib/Driver.php,v 1.11 2004/05/23 17:09:24 jan Exp $
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @version $Revision: 1.11 $
 * @since   Odin 0.0.1
 * @package Odin
 */
class Odin_Driver {

    /**
     * String holding the name of the current driver.
     * @var string $type
     */
    var $type = '';

    /**
     * Hash holding the driver's additional parameters.
     * @var array $_params
     */
    var $_params = array();

    /**
     * Our parent Odin_Source object.
     * @var object Odin_Source
     */
    var $_source;

    /**
     * Attempts to return a concrete Odin_Driver instance based on $driver.
     *
     * @param $driver   The type of Odin_Driver subclass to return.  The
     *                  code is dynamically included.
     * @param $params   Hash containing additional paramters to be passed to
     *                  the subclass's constructor.
     *
     * @return          The newly created concrete Odin_Driver instance, or
     *                  false on an error.
     */
    function &factory($driver, $params = array())
    {
        $driver = basename($driver);
        include_once dirname(__FILE__) . '/Driver/' . $driver . '.php';
        $class = 'Odin_Driver_' . $driver;
        if (class_exists($class)) {
            return $ret = &new $class($params);
        } else {
            Horde::fatal(PEAR::raiseError(sprintf(_("Unable to load the definition of %s."), $class)), __FILE__, __LINE__);
        }
    }

    /**
     * Attempts to return a reference to a concrete Odin_Driver instance
     * based on $driver. It will only create a new instance if no
     * Odin_Driver instance with the same parameters currently exists.
     *
     * This method must be invoked as: $var = &Odin_Driver::singleton()
     *
     * @param $driver   The type of concrete Odin_Driver subclass to return.
     *                  This is based on the storage driver ($driver). The
     *                  code is dynamically included.
     * @param $params   A hash containing additional parameters for the
     *                  subclass.
     *
     * @return          The concrete Odin_Driver reference, or false on an
     *                  error.
     */
    function &singleton($driver, $params)
    {
        static $instances;

        if (!isset($instances)) {
            $instances = array();
        }

        $signature = serialize(array($driver, $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &Odin_Driver::factory($driver, $params);
        }

        return $instances[$signature];
    }

    function setSource(&$source)
    {
        $this->_source = &$source;
    }

    /**
     * Returns the name of the current driver type.
     *
     * @return          String containing the name of the current driver type.
     */
    function getType()
    {
        return $this->type;
    }

    /**
     * Returns the current driver's additional parameters.
     *
     * @return          Hash containing the driver's additional parameters.
     */
    function getParams()
    {
        return $this->_params;
    }

    /**
     * Searches the backend with the given criteria and returns a
     * filtered list of results. If no criteria are specified, all
     * records are returned.
     *
     * @abstract
     *
     * @param $criteria      Array containing the search criteria.
     * @param $fields        List of fields to return.
     * @param $strict_fields (optional) Array of fields which must be matched
     *                       exactly.
     * @param const $match   Do an 'and' or an 'or' search (defaults to ODIN_SEARCH_AND).
     *
     * @return               Hash containing the search results.
     */
    function search($criteria, $fields, $strict_fields = array(), $match = ODIN_SEARCH_AND)
    {
    }

    /**
     * Read the given data from the backend and returns
     * the result's fields.
     *
     * @abstract
     *
     * @param $criteria      Search criteria.
     * @param $id      	     Data identifier.
     * @param $fields        List of fields to return.
     *
     * @return               Hash containing the search results.
     */
    function read($criteria, $id, $fields)
    {
    }

    /**
     * Adds the specified object to the backend.
     *
     * @abstract
     */
    function addObject($attributes)
    {
    }

    /**
     * Deletes the specified object from the backend.
     *
     * @abstract
     */
    function removeObject($object_key, $object_id)
    {
    }

    /**
     * Saves the specified object to the backend.
     *
     * @abstract
     */
    function setObject($attributes)
    {
    }

}
