<?php
/**
 * Odin data source driver implementation for PHP's PEAR database
 * abstraction layer.
 *
 * $Horde: odin/lib/Driver/sql.php,v 1.14 2004/10/08 15:45:25 chuck Exp $
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @version $Revision: 1.14 $
 * @since   Odin 0.0.1
 * @package Odin
 */
class Odin_Driver_sql extends Odin_Driver {

    /** Handle for the current database connection. */
    var $_db = null;

    /**
     * Constructs a new Odin SQL driver object.
     *
     * @param $params       Hash containing additional configuration parameters.
     */
    function Odin_Driver_sql($params)
    {
        $this->type = 'sql';
        $this->_params = $params;

        include_once 'DB.php';
        $this->_db = &DB::connect($params,
                                  array('persistent' => !empty($this->_params['persistent'])));
        if (!is_a($this->_db, 'PEAR_Error')) {
            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
        } else {
            $this->errno = -1;
            $this->errstr = $this->_db->getMessage();
        }
    }

    function runSummary($summary, $input = null)
    {
        if (!empty($summary['andOr']) && $summary['andOr'] == 'or') {
            $andOr = ' OR ';
        } else {
            $andOr = ' AND ';
        }

        $where = '';
        if (isset($summary['criteria']) && is_array($summary['criteria'])) {
            foreach ($summary['criteria'] as $field => $test) {
                if (is_array($test) && count($test)) {
                    $oper_where = '(';
                    foreach ($test as $oper => $term) {
                        if ($oper_where != '(') {
                            $oper_where .= $andOr;
                        }
                        if ($term == '%INPUT%') {
                            $term = $input;
                        }
                        $oper_where .= $this->getDriverKey($field) . ' ' . $oper . ' ' . $this->_db->quote($term);
                    }
                    if (!empty($where)) {
                        $where .= $andOr;
                    }
                    $where .= $oper_where . ')';
                }
            }
        }

        $fields = '';
        if (!is_array($summary['fields'])) {
            return PEAR::raiseError(_("Summaries must include at least 1 field."));
        }
        foreach ($summary['fields'] as $field) {
            $fields[] = $this->getDriverKey($field);
        }

        switch ($summary['operation']) {
        case 'sum':
        case 'count':
            $operation = strtoupper($summary['operation']);
            if (count($fields) > 1) {
                $select = '';
                foreach ($fields as $field) {
                    if (!empty($select)) {
                        $select .= ' + ';
                    }
                    $select .= $operation . '(' . $field . ')';
                }
            } else {
                $select = $operation . '(' . $fields[0] . ')';
            }
            break;
        }

        // Add any groupby fields to the select list, and build the
        // GROUP BY clause.
        $groupby = array();
        if (isset($summary['groupby'])) {
            foreach ($summary['groupby'] as $field) {
                $f = $this->getDriverKey($field);
                $groupby[] = $f;
                $select .= ', ' . $f;
            }
        }
        if (count($groupby)) {
            $groupby = ' GROUP BY ' . implode(', ', $groupby);
        } else {
            $groupby = '';
        }

        if (!empty($where)) {
            $where = ' WHERE ' . $where;
        }
        $query = sprintf('SELECT %s FROM %s%s%s',
                         $select,
                         $this->_params['table'],
                         $where,
                         $groupby);

        switch ($summary['type']) {
        case 'value':
            return $this->_db->getOne($query);

        case 'array':
            return $this->_db->getAssoc($query);
        }
    }

    /**
     * Searches the SQL database with the given criteria and returns a
     * filtered list of results. If the criteria parameter is an empty
     * array, all records will be returned.
     *
     * @param $criteria      Array containing the search criteria.
     * @param $fields        List of fields to return.
     * @param $strict_fields (optional) Array of fields which must be matched
     *                       exactly.
     * @param const $match   Do an 'and' or an 'or' search (defaults to ODIN_SEARCH_AND).
     *
     * @return               Hash containing the search results.
     */
    function search($criteria, $fields, $strict_fields = array(), $match = ODIN_SEARCH_AND)
    {
        if ($match === ODIN_SEARCH_AND) {
            $glue = ' AND ';
        } else {
            $glue = ' OR ';
        }

        $where = '';
        if (count($criteria) > 0) {
            /* Sort into strict and non-strict fields. */
            $strict = array();
            $loose = array();
            $strict_where = '';
            $loose_where = '';

            foreach ($criteria as $key => $val) {
                if (in_array($key, $strict_fields)) {
                    $strict[$key] = $val;
                } else {
                    $loose[$key] = $val;
                }
            }

            /* Add strict sub-query. */
            if (count($strict) > 0) {
                $strict_where = ' (';
                foreach ($strict as $key => $val) {
                    if ($strict_where != ' (') {
                        $strict_where .= ' AND ';
                    }
                    $strict_where .= $key . ' = ' . $this->_db->quote($val);
                }
                $strict_where .= ')';
            }

            /* Add loose sub-query. */
            if (count($loose) > 0) {
                $loose_where = ' (';
                foreach ($loose as $key => $val) {
                    if ($loose_where != ' (') {
                        $loose_where .= ' ' . $glue;
                    }
                    if (is_array($val)) {
                        if (count($val)) {
                            $oper_where = '(';
                            foreach ($val as $oper => $term) {
                                if ($oper_where != '(') {
                                    $oper_where .= ' AND ';
                                }
                                $oper_where .= $key . ' ' . $oper . ' ' . $this->_db->quote($term);
                            }
                            $loose_where .= $oper_where . ')';
                        }
                    } else {
                        $loose_where .= 'LOWER(' . $key . ') LIKE LOWER(' . $this->_db->quote('%' . $val . '%') . ')';
                    }
                }
                $loose_where .= ')';
            }

            /* Build full WHERE clause. */
            if (!empty($strict_where) || !empty($loose_where)) {
                $where = ' WHERE ';
                if (!empty($strict_where)) {
                    $where .= $strict_where;
                }
                if (!empty($strict_where) && !empty($loose_where)) {
                    $where .= ' AND ';
                }
                if (!empty($loose_where)) {
                    $where .= $loose_where;
                }
            }
        }

        $query  = 'SELECT ' . implode(', ', $fields) . ' ';
        $query .= 'FROM ' . $this->_params['table'];
        $query .= $where;

        /* Log the query at a DEBUG log level. */
        Horde::logMessage(sprintf('SQL search by %s: table = %s; query = "%s"',
                                  Auth::getAuth(), $this->_params['table'], $query),
                          __FILE__, __LINE__, PEAR_LOG_DEBUG);

        /* Run query. */
        $result = $this->_db->query($query);
        $results = array();
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        while ($row = $result->fetchRow()) {
            if (is_a($row, 'PEAR_Error')) {
                return $row;
            }

            $entry = array();
            for ($i = 0; $i < count($fields); $i++) {
                $field = $fields[$i];
                $entry[$field] = $row[$i];
            }

            $results[] = $entry;
        }

        return $results;
    }

    /**
     * Read the given data from the SQL database and returns
     * the result's fields.
     *
     * @param $criteria      Search criteria.
     * @param $id            Data identifier.
     * @param $fields        List of fields to return.
     *
     * @return               Hash containing the search results.
     */
    function read($criteria, $id, $fields)
    {
        $where = $criteria . ' = ' . $this->_db->quote($id) ;

        $query  = 'SELECT ' . implode(', ', $fields) . ' ';
        $query .= 'FROM ' . $this->_params['table'] . ' WHERE ' . $where;

        $result = $this->_db->query($query);

        $results = array();
        if (!is_a($result, 'PEAR_Error')) {
            $row = $result->fetchRow();
            if (isset($row) && !is_a($row, 'PEAR_Error')) {
                $entry = array();

                for ($i = 0; $i < count($fields); $i++) {
                    $field = $fields[$i];
                    $entry[$field] = $row[$i];
                }

                $results[] = $entry;
            }
        }

        return $results;
    }

    /**
     * Adds the specified object to the SQL database.
     */
    function addObject($attributes)
    {
        $fields = array();
        $values = array();
        foreach ($attributes as $field => $value) {
            $fields[] = $field;
            $values[] = $this->_db->quote($value);
        }

        $query  = 'INSERT INTO ' . $this->_params['table'] . ' (' . implode(', ', $fields) . ')';
        $query .= ' VALUES (' . implode(', ', $values) . ')';

        $result = $this->_db->query($query);
        return is_a($result, 'PEAR_Error') ? $result : true;
    }

    /**
     * Deletes the specified object from the SQL database.
     */
    function removeObject($object_key, $object_id)
    {
        $where = $object_key . ' = ' . $this->_db->quote($object_id);
        $query = 'DELETE FROM ' . $this->_params['table'] . ' WHERE ' . $where;

        $result = $this->_db->query($query);

        return is_a($result, 'PEAR_Error') ? $result : true;
    }

    /**
     * Saves the specified object in the SQL database.
     */
    function setObject($object_key, $object_id, $attributes)
    {
        $where = $object_key . ' = ' . $this->_db->quote($object_id);
        unset($attributes[$object_key]);

        $set = array();
        foreach ($attributes as $field => $value) {
            $set[] = $field . ' = ' . $this->_db->quote($value);
        }

        $query  = 'UPDATE ' . $this->_params['table'] . ' SET ' . implode(', ', $set) . ' ';
        $query .= 'WHERE ' . $where;

        $result = $this->_db->query($query);

        return is_a($result, 'PEAR_Error') ? $result : true;
    }

    /**
     * Create an object key for a new object.
     *
     * @param array $attributes  The attributes (in driver keys) of the
     *                           object being added.
     *
     * @return string  A unique ID for the new object.
     */
    function makeKey($attributes)
    {
        return md5(mt_rand());
    }

    function getDriverKey($key)
    {
        list($return,) = each($this->_source->toDriverKeys(array($key => '')));
        return $return;
    }

}
