<?php
/**
 * The Odin_List:: class provides an interface for dealing with a
 * list of Odin_Objects.
 *
 * $Horde: odin/lib/List.php,v 1.8 2004/05/23 17:09:24 jan Exp $
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @author  Jon Parise <jon@csh.rit.edu>
 * @version $Revision: 1.8 $
 * @since   Odin 0.0.1
 * @package Odin
 */
class Odin_List {

    /** The array containing the Odin_Objects represented in this
        list. */
    var $objects = array();

    /** An array of objects which have just been added during this
        page load. */
    var $fresh = array();

    /**
     * Have we already raised a full alert?
     * @var boolean $_full
     */
    var $_full = false;

    /**
     * Constructs a new Odin_List object.
     */
    function Odin_List()
    {
    }

    /**
     * Inserts a new list.
     *
     * @param          object Odin_Object $object  The object to insert.
     * @param optional boolean             $new     This object is from a new search (defaults to true).
     */
    function insert($object, $new = true)
    {
        global $conf, $notification;

        if ($this->count() >= $conf['search']['max_results']) {
            if (!$this->_full) {
                $this->_full = true;
                $notification->push(sprintf(_("There are too many search results; only displaying %s. Please narrow your search."), $conf['search']['max_results']), 'horde.warning');
            }
            return PEAR::raiseError(_("Too many results"));
        }

        if (is_object($object)) {
            $key = $object->source->name . ':' . $object->getValue('__key');
            if (!isset($this->objects[$key])) {
                if ($new) {
                    $this->fresh[$key] = 1;
                }
                $this->objects[$key] = $object;
            }
        }

        return true;
    }

    /**
     * Remove an entry from the list.
     *
     * @param string $key  The key of the object to remove.
     *
     * @since Odin 1.1.1
     */
    function remove($key)
    {
        if (isset($this->objects[$key])) {
            unset($this->objects[$key]);
        }
    }

    /**
     * Merges an existing Odin_List into this one.
     *
     * @param          mixed   $list  The list to merge - either a Odin_List object or an array.
     * @param optional boolean $new   These objects are coming from a new search (defaults to true).
     */
    function merge($list, $new = true)
    {
        if (!$this->isFull()) {
            if (is_object($list)) {
                $list = $list->objects;
            }
            if (is_array($list)) {
                foreach ($list as $object) {
                    $result = $this->insert($object, $new);
                    if (is_a($result, 'PEAR_Error')) {
                        return $result;
                    }
                }
            }
        }
    }

    /**
     * Reset our internal pointer to the beginning of the list. Use
     * this to hide the internal storage (array, list, etc.) from
     * client objects.
     */
    function reset()
    {
        reset($this->objects);
    }

    /**
     * Return the next Odin_Object in the list. Use this to hide
     * internal implementation details from client objects.
     *
     * @return Odin_Object $object   The next object in the list.
     */
    function next()
    {
        list(,$tmp) = each($this->objects);
        return $tmp;
    }

    /**
     * Return the number of Odin_Objects that are in the list. Use
     * this to hide internal implementation details from client
     * objects.
     *
     * @return integer $count The number of objects in the list.
     */
    function count()
    {
        return count($this->objects);
    }

    /**
     * Check if the list has already overflowed.
     *
     * @return boolean  True if yes, false if not yet full.
     */
    function isFull()
    {
        return $this->_full;
    }

    /**
     * Filters/Sorts the list based on the specified sort routine.
     *
     * @param $sort         The sort method.
     * @param $low          The low end of the sort range.
     * @param $high         The high end of the sort range.
     * @param $dir          Sort direction, 0 = ascending, 1 = descending
     */
    function sort($sort = 'lastname', $dir = 0)
    {
        $sorted_objects = array();

        foreach ($this->objects as $key => $object) {
            $lastname = $this->guessLastname($object->getValue('name'));
            $object->setValue('lastname', $lastname);
            $sorted_objects[$key] = $object;
        }

        $GLOBALS['usortCriteria'] = $sort;
        usort($sorted_objects, array($this, 'cmp'));

        if ($dir == 1) {
            $this->objects = array_reverse($sorted_objects);
        } else {
            $this->objects = $sorted_objects;
        }
    }

    /**
     * Returns a best guess at the lastname in a string.
     *
     * @param $name     String contain the full name.
     *
     * @return          String containing the last name.
     */
    function guessLastname($name)
    {
        if (!empty($name)) {
            /* Assume that last names are always before any commas. */
            if (is_int(strpos($name, ','))) {
                $name = String::substr($name, 0, String::pos($name, ','));
            }

            /* Take out anything in parentheses. */
            $name = preg_replace('|\(.*\)|', '', $name);

            /* Remove single quotes. */
            $name = str_replace('\'', '', $name);

            $name = trim(preg_replace('|\s+|', ' ', $name));
            $namelist = explode(' ', $name);
            $name = $namelist[($nameindex = (count($namelist) - 1))];

            while (strlen($name) < 5 &&
                   strspn($name[(strlen($name) - 1)], '.:-') &&
                   isset($namelist[($nameindex - 1)])) {
                $nameindex--;
                $name = $namelist[$nameindex];
            }
        }
        return $name;
    }

    /**
     * Usort helper function. Compares two Odin_Objects
     * based on the global variable $usortCriteria.
     *
     * @param $a        The first Odin_Object to compare.
     * @param $b        The second Odin_Object to compare.
     *
     * @return          Integer comparison of the two addresses from strcmp().
     */
    function cmp($a, $b)
    {
        global $usortCriteria;

        $acmp = strtolower($a->getValue($usortCriteria));
        $bcmp = strtolower($b->getValue($usortCriteria));

        return strcmp($acmp, $bcmp);
    }

    function isFresh($object)
    {
        return isset($this->fresh[$object->source->name . ':' . $object->getValue('__key')]);
    }

    function serialize()
    {
        $data = array();
        $data['keys'] = array();
        $data['fresh'] = $this->fresh;
        foreach ($this->objects as $object) {
            $data['keys'][] = $object->source->name . ':' . $object->getValue('__key');
        }
        return $data;
    }

    function unserialize($data)
    {
        if (!isset($data) || !is_array($data)) {
            return null;
        }

        $tmp = new Odin_List();
        $objects = array();
        foreach ($data['keys'] as $value) {
            list($source, $key) = explode(':', $value);
            if (isset($GLOBALS['cfgSources'][$source])) {
                $sourceOb = &Odin_Source::singleton($source, $GLOBALS['cfgSources'][$source]);
                $result = $tmp->insert($sourceOb->getObject($key));
                if (is_a($result, 'PEAR_Error')) {
                    return $result;
                }
            }
        }

        /* Not the best way of doing this, but it works for now */
        $tmp->fresh = $data['fresh'];
        return $tmp;
    }

}
